@@ -35,7 +35,6 @@ create_color_pixbuf (const char *color)
 
         int x;
         int num;
-        int rowstride;
         guchar *pixels, *p;
 
         if (!gdk_color_parse (color, &col))
@@ -45,16 +44,15 @@ create_color_pixbuf (const char *color)
                                  FALSE, 8,
                                  16, 16);
 
-        rowstride = gdk_pixbuf_get_rowstride (pixbuf);
         p = pixels = gdk_pixbuf_get_pixels (pixbuf);
 
         num = gdk_pixbuf_get_width (pixbuf) *
                 gdk_pixbuf_get_height (pixbuf);
 
         for (x = 0; x < num; x++) {
-                p[0] = col.red / 65535 * 255;
-                p[1] = col.green / 65535 * 255;
-                p[2] = col.blue / 65535 * 255;
+                p[0] = col.red / 65535.0 * 255;
+                p[1] = col.green / 65535.0 * 255;
+                p[2] = col.blue / 65535.0 * 255;
                 p += 3;
         }
 
@@ -70,21 +68,15 @@ create_combo_box_grid_demo (void)
         GtkCellRenderer *cell = gtk_cell_renderer_pixbuf_new ();
         GtkListStore *store;
 
-        store = gtk_list_store_new (1, GDK_TYPE_PIXBUF);
-
-        combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
-        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
-                                    cell, TRUE);
-        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo),
-                                        cell, "pixbuf", 0, NULL);
-        gtk_combo_box_set_wrap_width (GTK_COMBO_BOX (combo),
-                                      3);
+        store = gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_INT, G_TYPE_INT);
 
         /* first row */
         pixbuf = create_color_pixbuf ("red");
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, pixbuf,
+                            1, 1, /* row span */
+                            2, 1, /* column span */
                             -1);
         g_object_unref (pixbuf);
 
@@ -92,6 +84,8 @@ create_combo_box_grid_demo (void)
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, pixbuf,
+                            1, 1,
+                            2, 1,
                             -1);
         g_object_unref (pixbuf);
 
@@ -99,6 +93,8 @@ create_combo_box_grid_demo (void)
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, pixbuf,
+                            1, 1,
+                            2, 1,
                             -1);
         g_object_unref (pixbuf);
 
@@ -107,6 +103,8 @@ create_combo_box_grid_demo (void)
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, pixbuf,
+                            1, 1,
+                            2, 2, /* Span 2 columns */
                             -1);
         g_object_unref (pixbuf);
 
@@ -114,13 +112,8 @@ create_combo_box_grid_demo (void)
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, pixbuf,
-                            -1);
-        g_object_unref (pixbuf);
-
-        pixbuf = create_color_pixbuf ("white");
-        gtk_list_store_append (store, &iter);
-        gtk_list_store_set (store, &iter,
-                            0, pixbuf,
+                            1, 2, /* Span 2 rows */
+                            2, 1,
                             -1);
         g_object_unref (pixbuf);
 
@@ -129,13 +122,8 @@ create_combo_box_grid_demo (void)
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, pixbuf,
-                            -1);
-        g_object_unref (pixbuf);
-
-        pixbuf = create_color_pixbuf ("snow");
-        gtk_list_store_append (store, &iter);
-        gtk_list_store_set (store, &iter,
-                            0, pixbuf,
+                            1, 1,
+                            2, 1,
                             -1);
         g_object_unref (pixbuf);
 
@@ -143,11 +131,25 @@ create_combo_box_grid_demo (void)
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             0, pixbuf,
+                            1, 1,
+                            2, 1,
                             -1);
         g_object_unref (pixbuf);
 
+        /* Create ComboBox after model to avoid gtk_menu_attach() warnings(?) */
+        combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
         g_object_unref (store);
 
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo),
+                                    cell, TRUE);
+        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo),
+                                        cell, "pixbuf", 0, NULL);
+
+        /* Set wrap-width != 0 to enforce grid mode */
+        gtk_combo_box_set_wrap_width (GTK_COMBO_BOX (combo), 3);
+        gtk_combo_box_set_row_span_column (GTK_COMBO_BOX (combo), 1);
+        gtk_combo_box_set_column_span_column (GTK_COMBO_BOX (combo), 2);
+
         gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
         return combo;
@@ -1021,6 +1023,7 @@ main (int argc, char **argv)
 	GtkTreePath *path;
 	GtkTreeIter iter;
         GdkColor color;
+        gint i;
 
         gtk_init (&argc, &argv);
 
@@ -1331,6 +1334,18 @@ main (int argc, char **argv)
 	gtk_tree_path_free (path);
         gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combobox), &iter);
 
+        tmp = gtk_frame_new ("Looong");
+        gtk_box_pack_start (GTK_BOX (mainbox), tmp, FALSE, FALSE, 0);
+        combobox = gtk_combo_box_text_new ();
+        for (i = 0; i < 200; i++)
+          {
+            gchar *text = g_strdup_printf ("Item %d", i);
+            gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combobox), text);
+            g_free (text);
+          }
+        gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), 53);
+        gtk_container_add (GTK_CONTAINER (tmp), combobox);
+
 #if 1
 	gdk_threads_add_timeout (1000, (GSourceFunc) capital_animation, model);
 #endif
